@@ -18,7 +18,8 @@ __all__ = [
     "tasks",
     "utils",
     "training",
-    "optimizers"
+    "optimizers",
+    "sparsity"
 ]
 
 importlib.import_module("neurst.criterions")
@@ -34,11 +35,13 @@ importlib.import_module("neurst.layers.attentions")
 importlib.import_module("neurst.layers.decoders")
 importlib.import_module("neurst.layers.encoders")
 importlib.import_module("neurst.layers.metric_layers")
+importlib.import_module("neurst.layers.quantization")
 importlib.import_module("neurst.layers.search")
 importlib.import_module("neurst.metrics")
 importlib.import_module("neurst.models")
 importlib.import_module("neurst.optimizers")
 importlib.import_module("neurst.optimizers.schedules")
+importlib.import_module("neurst.sparsity")
 importlib.import_module("neurst.tasks")
 importlib.import_module("neurst.training")
 importlib.import_module("neurst.utils")
