@@ -222,7 +222,8 @@ def load_from_tfrecord_and_auto_shard(features_file, shuffle=True,
 
 
 def parse_tfexample(serialized_example, name_to_features,
-                    name_mapping=None, map_func=None):
+                    name_mapping=None, map_func=None,
+                    auxiliary_elements=None):
     """ Parses TF example from TF Record. """
     parsed = tf.io.parse_single_example(serialized_example, name_to_features)
     elements = {}
@@ -231,6 +232,9 @@ def parse_tfexample(serialized_example, name_to_features,
             elements[k] = tf.sparse.to_dense(v)
         else:
             elements[name_mapping[k]] = tf.sparse.to_dense(v)
+    if isinstance(auxiliary_elements, dict):
+        elements.update(auxiliary_elements)
+
     if map_func is None:
         return elements
     return map_func(elements)
@@ -257,7 +261,8 @@ def load_tfrecords(file_path,
                    map_func=None,
                    sharding_index=0,
                    num_shards=1,
-                   auto_shard=False) -> tf.data.Dataset:
+                   auto_shard=False,
+                   auxiliary_elements=None) -> tf.data.Dataset:
     """ Loads TFRecords and does autot-sharding according to worker num.
 
     Args:
@@ -271,22 +276,37 @@ def load_tfrecords(file_path,
         sharding_index: The manually defined index for sharding.
         num_shards: The manually defined number of shards operating in parallel.
         auto_shard: Automatically shard the TFRecord parts if True.
+        auxiliary_elements: A dict containing auxiliary elements that will
+            append to the data sample.
 
     Returns: A dataset.
     """
-    _features_files = glob_tfrecords(file_path)
-    shuffle = shuffle and (num_shards == 1)
-    dataset = tf.data.Dataset.list_files(_features_files, shuffle=shuffle)
+    _features_files = []
+    for _file in flatten_string_list(file_path):
+        if tf.io.gfile.isdir(_file):
+            _features_files.append(os.path.join(_file, "*train*"))
+        elif tf.io.gfile.exists(_file):
+            _features_files.append(_file)
+        else:
+            _features_files.append(_file + "*")
+    # shuffle = (shuffle is True) and (num_shards == 1)
+    # dataset = tf.data.Dataset.list_files(_features_files, shuffle=shuffle)
+    dataset = tf.data.Dataset.list_files(_features_files, shuffle=False)
     if num_shards > 1:
         logging.info("Shard %d of the whole dataset(total %d workers).", sharding_index, num_shards)
         dataset = dataset.shard(num_shards, sharding_index)
     else:
         # auto sharding
         worker_id, num_workers, strategy = get_distributed_worker_setting()
-        if num_workers > 1 and strategy in ["horovod", "byteps"] and not shuffle and auto_shard:
+        if num_workers > 1 and strategy in ["horovod", "byteps"] and auto_shard:
             logging.info("Shard %d of the whole dataset(total %d workers).", worker_id, num_workers)
+            options = tf.data.Options()
+            options.experimental_distribute.auto_shard_policy = tf.data.experimental.AutoShardPolicy.OFF
+            dataset = dataset.with_options(options)
             dataset = dataset.shard(num_workers, worker_id)
     logging.info("Loading TF Records from: ")
+    if shuffle:
+        dataset = dataset.shuffle(5000)
     for _f in dataset:
         logging.info(f"   {_f.numpy()}")
     # Read files and interleave results.
@@ -300,7 +320,8 @@ def load_tfrecords(file_path,
 
     if name_to_features is None:
         return dataset
-    return dataset.map(lambda x: parse_tfexample(x, name_to_features, feature_name_mapping, map_func),
+    return dataset.map(lambda x: parse_tfexample(x, name_to_features, feature_name_mapping, map_func,
+                                                 auxiliary_elements=auxiliary_elements),
                        num_parallel_calls=tf.data.experimental.AUTOTUNE)
 
 
@@ -524,3 +545,22 @@ def batch_examples_by_token(dataset,
         reduce_func=batching_fn,
         window_size=None,
         window_size_func=window_size_fn))
+
+
+def take_one_record(data_path):
+    _file_path = flatten_string_list(data_path)[0]
+    if tf.io.gfile.isdir(_file_path):
+        _feature_file = os.path.join(_file_path, "*train*")
+    elif tf.io.gfile.exists(_file_path):
+        _feature_file = _file_path
+    else:
+        _feature_file = _file_path + "*"
+    dataset = tf.data.Dataset.list_files([_feature_file], shuffle=False)
+    dataset = dataset.interleave(
+        lambda f: tf.data.TFRecordDataset(f, buffer_size=128 * 1024 * 1024),
+        cycle_length=10,
+        num_parallel_calls=tf.data.experimental.AUTOTUNE)
+    for x in dataset.take(1):
+        example = tf.train.Example()
+        example.ParseFromString(x.numpy())
+        return example
