@@ -352,7 +352,7 @@ def _count_tokens(files, file_byte_limit=1e6):
                 else:
                     if file_byte_budget < 0:
                         break
-                    line = line.strip()
+                    line = " ".join(line.strip().split())
                     file_byte_budget -= len(line)
                     counter = 0
 
@@ -411,7 +411,7 @@ def _escape_token(token, alphabet):
     return u"".join(ret) + "_"
 
 
-@register_tokenizer("sub_tokenizer")
+@register_tokenizer(["sub_tokenizer", "word_piece", "wordpiece"])
 class Subtokenizer(Tokenizer):
 
     def __init__(self, language, glossaries=None, **kwargs):
@@ -432,17 +432,21 @@ class Subtokenizer(Tokenizer):
 
     def init_subtokenizer(self, codes):
         subtoken_list = []
-        with tf.io.gfile.GFile(codes, mode="r") as f:
-            for line in f:
-                subtoken = line.strip()
-                if ((subtoken.startswith("'") and subtoken.endswith("'"))
-                    or (subtoken.startswith('"') and subtoken.endswith('"'))):
-                    subtoken = subtoken[1:-1]  # Remove surrounding single-quotes
-                if subtoken in self._reserved_tokens:
-                    continue
-                if " " in subtoken:
-                    self._space_wrapper = None
-                subtoken_list.append(subtoken)
+        if isinstance(codes, list):
+            subtokens = codes
+        else:
+            with tf.io.gfile.GFile(codes, mode="r") as f:
+                subtokens = [line for line in f]
+        for subtoken in subtokens:
+            subtoken = subtoken.strip()
+            if ((subtoken.startswith("'") and subtoken.endswith("'"))
+                or (subtoken.startswith('"') and subtoken.endswith('"'))):
+                subtoken = subtoken[1:-1]  # Remove surrounding single-quotes
+            if subtoken in self._reserved_tokens:
+                continue
+            if " " in subtoken:
+                self._space_wrapper = None
+            subtoken_list.append(subtoken)
         self._all_subtoken_strings = subtoken_list + self._reserved_tokens
         # we remember the maximum length of any subtoken to avoid having to
         # check arbitrarily long strings.
