@@ -21,7 +21,8 @@ from neurst.data import dataset_utils
 from neurst.data.data_pipelines import DataPipeline, build_data_pipeline
 from neurst.data.data_pipelines.transcript_data_pipeline import TranscriptDataPipeline
 from neurst.data.datasets import Dataset
-from neurst.layers.metric_layers.token_metric_layers import AudioFramesMetricLayer, SequenceTokenMetricLayer
+from neurst.layers.metric_layers.token_metric_layers import (AudioFramesMetricLayer, BatchCountMetricLayer,
+                                                             SequenceTokenMetricLayer)
 from neurst.metrics import build_metric
 from neurst.models import build_model
 from neurst.models.model_utils import deduce_text_length
@@ -87,8 +88,7 @@ class SpeechToText(Task):
         this_args = super(SpeechToText, SpeechToText).class_or_method_args()
         this_args.extend([
             ModuleFlag("transcript_data_pipeline", DataPipeline.REGISTRY_NAME,
-                       default=TranscriptDataPipeline.__name__,
-                       help="The target side transcript data pipeline."),
+                       default=None, help="The target side transcript data pipeline."),
             Flag("audio_feature_dim", dtype=Flag.TYPE.INTEGER, default=80,
                  help="The dimension of audio features."),
             Flag("audio_feature_channels", dtype=Flag.TYPE.INTEGER, default=1,
@@ -202,7 +202,7 @@ class SpeechToText(Task):
             else:
                 assert data_status["transcript"] == compat.DataStatus.PROJECTED
             if mode == compat.ModeKeys.TRAIN and trunc_trg and max_trg_len:
-                if isinstance(text, tf.Tensor):
+                if compat.is_tf_tensor(text):
                     text = tf.cond(
                         tf.less_equal(tf.size(text), max_trg_len), lambda: text,
                         lambda: tf.concat([text[:(max_trg_len - 1)], text[-1:]], axis=0))
@@ -215,7 +215,7 @@ class SpeechToText(Task):
             feature = _process_audio(data["audio"])
             ret = {"audio": feature,
                    "audio_length": tf.cast(
-                       (tf.shape(feature)[0] if isinstance(feature, tf.Tensor)
+                       (tf.shape(feature)[0] if compat.is_tf_tensor(feature)
                         else feature.shape[0]) // self._audio_feature_dim // self._audio_feature_channels,
                        dtype=tf.int64)}
             if with_label:
@@ -378,10 +378,14 @@ class SpeechToText(Task):
                 window_size_func=window_size_fn))
 
     def build_metric_layer(self):
-        return [AudioFramesMetricLayer("src"), SequenceTokenMetricLayer("trg")]
+        return [AudioFramesMetricLayer("src"), SequenceTokenMetricLayer("trg"),
+                BatchCountMetricLayer("src")]
 
-    def get_eval_metric(self, args, name="metric"):
+    def get_eval_metric(self, args, name="metric", ds=None):
         """ Returns a neurst.metrics.metric.Metric object for evaluation."""
+        if ds is not None and hasattr(ds, "trg_lang") and ds.trg_lang is not None:
+            return build_metric(args[name + ".class"], language=ds.trg_lang,
+                                **args[name + ".params"])
         return build_metric(args[name + ".class"], language=self._trg_data_pipeline.meta["language"],
                             **args[name + ".params"])
 
